#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

/*
 * Global option flags
 */
int	Debug = 0;
int	Kilos = 0;

void
debug(int level, char *fmt, ...)
{
	va_list ap;

	if (Debug < level)
		return;
	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
}

int
error(int fatal, char *fmt, ...)
{
	va_list ap;

	fprintf(stderr, fatal ? "Error: " : "Warning: ");
	if (errno)
	    fprintf(stderr, "%s: ", strerror(errno));
	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
	if (fatal > 0)
	    exit(fatal);
	else
	{
	    errno = 0;
	    return (fatal);
	}
}

void
usage(void)
{
	fprintf(stderr,
"Usage:\n"
"	usb_throttle [options] kilobytes-per-second\n"
"\n"
"	Throttle USB output.\n"
"\n"
"Options:\n"
"       -D lvl      Set Debug level [%d]\n"
	, Debug
	);

	exit(1);
}

int
main(int argc, char *argv[])
{
	extern int	optind;
	extern char	*optarg;
	int		c;
	int		kilos;
	int		per10msec;
	size_t		size;
	char		*buf;

	while ( (c = getopt(argc, argv, "D:?h")) != EOF)
		switch (c)
		{
		case 'D':
			Debug = atoi(optarg);
			break;
		default:
			usage();
			exit(1);
		}

	argc -= optind;
	argv += optind;

	if (argc != 1)
	    usage();

	kilos = atoi(argv[0]);
	if (kilos < 1)
	    error(1, "Kilobytes-per-second must be greater than 0\n");
	debug(1, "kilos = %d\n", kilos);

	per10msec = (kilos * 1024) / 100;
	debug(1, "per10msec = %d\n", per10msec);

	buf = malloc(per10msec);
	if (!buf)
	    error(1, "Couldn't allocate %d bytes.\n", per10msec);

	while (1)
	{
	    usleep(10*1000);	
	    size = fread(buf, 1, per10msec, stdin);
	    if (size == 0)
		break;
	    fwrite(buf, 1, size, stdout);
	    fflush(stdout);
	    if (size < per10msec)
		break;
	}

	exit(0);
}
